"use strict";
exports.__esModule = true;
var defaults_1 = require("./defaults");
var utils_1 = require("./utils");
var path_1 = require("path");
var fs_1 = require("fs");
var lodash_1 = require("lodash");
var chokidar = require("chokidar");
exports.inputProvider = function (options, store) {
    var buildFilter = options.buildFilter, onSet = options.onSet, root = options.root;
    return function build(pathname, withbuilding) {
        if (pathname === void 0) { pathname = ''; }
        if (withbuilding === void 0) { withbuilding = true; }
        pathname = utils_1.fixPath(pathname);
        var absolutePath = path_1.join(root, pathname);
        var stats = fs_1.statSync(absolutePath);
        var fn = defaults_1.thenFn();
        if (pathname && (!buildFilter || !buildFilter(pathname))) {
            fn = defaults_1.thenFn();
        }
        else if (stats.isDirectory()) {
            fn = function (resolve, reject) {
                onSet(pathname, {}, store).then(function (_a) {
                    var data = _a.data, outputPath = _a.outputPath;
                    store._set(outputPath, {});
                    Promise.all(fs_1.readdirSync(absolutePath).map(function (filename) { return build(path_1.join(pathname, filename), false); }))
                        .then(resolve)["catch"](defaults_1.catchFn(reject));
                })["catch"](defaults_1.catchFn(reject));
            };
        }
        else if (stats.isFile()) {
            fn = function (resolve, reject) {
                fs_1.readFile(absolutePath, function (err, data) {
                    onSet(pathname, data, store).then(function (_a) {
                        var data = _a.data, outputPath = _a.outputPath;
                        store._set(outputPath, data);
                        resolve({ data: data, outputPath: outputPath });
                    })["catch"](defaults_1.catchFn(reject));
                });
            };
        }
        var res = new Promise(fn);
        //  BUGS: remove
        if (withbuilding) {
            store.setBuilding(1);
            var timeout_1 = setTimeout(function () {
                store.setBuilding(-1);
            }, 5000);
            res.then(function () {
                clearTimeout(timeout_1);
                store.setBuilding(-1);
            })["catch"](function () {
                clearTimeout(timeout_1);
                store.setBuilding(-1);
            });
        }
        return res;
    };
};
var inputProviderWithWatcher = function (options, store) {
    var build = exports.inputProvider(options, store);
    var buildWatcher = options.buildWatcher, root = options.root, watch = options.watch;
    if (watch && buildWatcher) {
        var watcher_1 = lodash_1.debounce(function (filename, eventType) {
            switch (eventType) {
                case 'change':
                    build(filename);
                    break;
            }
            buildWatcher(filename, eventType, build, store);
        }, 300);
        chokidar.watch(root, {
            ignored: defaults_1.REG.ignored
        }).on('all', function (eventType, filename) { return watcher_1(utils_1.fixPath(utils_1.fixPath(filename).replace(utils_1.fixPath(root), '')), eventType); });
    }
    return build;
};
exports["default"] = inputProviderWithWatcher;
